@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: otags.ml,v 1.32 2012-05-22 07:38:36 tews Exp $
+ * $Id: otags.ml,v 1.33 2012/12/05 12:18:16 tews Exp $
  * 
  * main module with main function
  * 
@@ -91,10 +91,12 @@ let read_camlp4_dump ic file unit =
 	  | Structure -> Struct_ast(input_value ic : str_item_t)
       in
       (line_directives, comp_unit)
-    else 
-      raise Skip_entry
+    else begin
+      Printf.eprintf 
+	"External camlp4 has incompatible version. Reinstall otags.\n";
+      exit 2
+    end
   with
-    | Skip_entry 
     | End_of_file ->
       Printf.eprintf "Error while reading the camlp4 ast for %s\n" file;
       raise Skip_entry
