@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: parser_hints.ml,v 1.1 2012/05/21 09:29:29 tews Exp $
+ * $Id: parser_hints.ml,v 1.2 2016/01/10 13:27:03 tews Exp $
  * 
  * parser hints functionality
  * 
@@ -109,7 +109,8 @@ and process_parser_hint_dir dir =
 	else process_parser_hint (Filename.concat dir entry)
       | None ->
 	not_finished := false
-  done
+  done;
+  U.closedir handle
 	
 
 let process_parser_hints =
