@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: tags.ml,v 1.29 2012/12/05 09:29:31 tews Exp $
+ * $Id: tags.ml,v 1.32 2016/01/08 21:46:24 tews Exp $
  * 
  * recursive tagging function
  * 
@@ -38,6 +38,25 @@ let empty_sig_ast = Sig_ast(<:sig_item@no_loc< >>)
 let empty_str_ast = Struct_ast(<:str_item@no_loc< >>)
 
 
+(******************************************************************************
+ *
+ **********************   utility functions   *********************************
+ *
+ ******************************************************************************)
+
+
+(** returns the first string of ident, return something arbitrary if 
+    the first element of the ident is an antiquotation.
+*)
+let rec first_string_of_ident = function
+  | <:ident< $x$ . $_$ >>
+  | <:ident< $x$ $_$ >> -> first_string_of_ident x
+  | <:ident< $lid:x$ >> -> x
+  | <:ident< $uid:x$ >> -> x
+  | <:ident< $anti:_$ >> -> "xxx"
+			     
+
+
 (******************************************************************************
  *
  **********************   tagging functions   *********************************
@@ -101,6 +120,13 @@ let rec tag_type write_tag typ = match typ with
       | _ -> assert false
     );
     tag_type write_tag type_def
+  | Ast.TyExt(_loc, id, _param_list, type_def) ->
+     (* For wellformed input, id is just a lowercase ident. I make
+      * this work even if id is something else (and the input is invalid).
+      *)
+     write_tag (translate_loc (Ast.loc_of_ident id))
+	       (first_string_of_ident id);
+     tag_type write_tag type_def
   | <:ctyp< $_t1$ == $t2$ >> -> 
     (* don't tag t1, its the abbreviation *)
     tag_type write_tag t2
@@ -113,6 +139,14 @@ let rec tag_type write_tag typ = match typ with
     List.iter
       (tag_record_label_decl write_tag)
       (Ast.list_of_ctyp t [])
+  | Ast.TyAtt(_loc, _attr, _payload, t) ->
+     (* In 4.02 camlp4 does not parse item attributes "[@@foo]" that can be
+      * attachted to whole type definitions. In the future they may appear in
+      * t as TyAtt node wrapped around a variant type. Therefore I recurse
+      * here. In 4.02 t may contain a TyAtt, but only for type equations 
+      * (type a = int [@foo]), for which recursion would not be necessary.
+      *)
+     tag_type write_tag t
 
   | <:ctyp< $tup:_$ >>			(* tuple *)
   | <:ctyp< [= $_$ ] >>			(* exact variant type *)
@@ -134,6 +168,7 @@ let rec tag_type write_tag typ = match typ with
   | <:ctyp< $_$ -> $_$ >>
   | <:ctyp< $_$ $_$ >>			(* type constructor application *)
   | <:ctyp< $_$ as $_$ >>
+  | <:ctyp< .. >>			(* empty extensible type *)
   | <:ctyp< >>
     -> ()
 
@@ -254,7 +289,13 @@ and tag_class_type write_tag = function
     List.iter
       (tag_class_sig_item write_tag)
       (Ast.list_of_class_sig_item ci [])
-
+  | Ast.CtAtt(_loc, _attr, _payload, ct) ->
+     (* AFAICT there is no camlp4 grammar production constructing CtAtt.
+      * Nevertheless, I would expect CtAtt to be wrapped around class types as
+      * in "class ca = (object end : object end [@foo])", therefore I recurse.
+      *)
+     tag_class_type write_tag ct
+     
   | <:class_type< $virtual:_$ $id:_$ [ $_$] >> 
     (* class id somewhere on the right hand side *)
     -> ()
@@ -285,7 +326,10 @@ let rec tag_sig_item write_tag sig_item = match sig_item with
     write_tag id_loc id
   | <:sig_item< module $id$ : $mt$ >> -> 
     (* missing location info for the identifier: See #5147.
-     * we need the second word 
+     * we need the second word
+     *
+     * This case does also match module aliases "module A = B", 
+     * the module type is then a module alias.
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_sig_item sig_item) in
     write_tag (Reparse.loc_of_second_word wrong_id_loc) id;
@@ -311,6 +355,12 @@ let rec tag_sig_item write_tag sig_item = match sig_item with
      * for infix identifiers.
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_sig_item sig_item) in
+    (* 
+     * Printf.eprintf "SIGVAL %s %s -> %s\n%!" 
+     *   id
+     *   (Source_channel.full_string_of_loc (Ast.loc_of_sig_item sig_item))
+     *   (Source_channel.full_string_of_loc wrong_id_loc);
+     *)
     let loc_without_val = Reparse.loc_without_first_word wrong_id_loc in
     let id_loc = match Reparse.start_char loc_without_val with
       | '(' -> Reparse.loc_inside_parens loc_without_val
@@ -330,7 +380,7 @@ let rec tag_sig_item write_tag sig_item = match sig_item with
       (tag_module_sig_binding write_tag)
       (Ast.list_of_module_binding mb [])
 
-  | <:sig_item< open $_$ >> 
+  | Ast.SgOpn _ 			(* open x, open! x *)
   | <:sig_item< include $_$ >>
     -> ()
 
@@ -348,6 +398,7 @@ and tag_module_sig_binding write_tag mod_binding = match mod_binding with
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_module_binding mod_binding) in
     write_tag (Reparse.loc_of_first_word wrong_id_loc) id;
+
     tag_module_type write_tag mtyp
 
   | <:module_binding< >>
@@ -372,10 +423,23 @@ and tag_module_type write_tag = function
     List.iter 
       (tag_sig_item write_tag)
       (Ast.list_of_sig_item sig_items [])
+  | Ast.MtAtt(_loc, _attr, _payload, mt) ->
+     (* AFAICT there is no camlp4 grammar production constructing MtAtt.
+      * Nevertheless, I would expect MtAtt to be wrapped around signature
+      * types as in "module type A = sig end [@foo]", therefore I recurse.
+      *)
+     tag_module_type write_tag mt
 
   | <:module_type< >>			(* emty module type ast *)
   | <:module_type< ' $_$ >>		(* ??? MtQuo ??? *)
   | <:module_type< module type of $_$ >>
+  | <:module_type< (module $id:_$) >>
+      (* module alias as module type; for a module alias in signatures 
+       * "module A = B", the A actually matches 
+       * <:sig_item< module $id$ : $mt$ >> and the mt then matches this 
+       * case. Camlp4 also accepts "functor (A : (module B)) -> ..." 
+       * but ocaml does not.
+       *)
     -> ()
 
   | <:module_type< $anti:_$ >>		(* anti quotations *)
@@ -487,6 +551,12 @@ and tag_class_expr write_tag = function
     tag_class_expr write_tag ce;
     (* ct can contain an object body, therefore tag it *)
     tag_class_type write_tag ct
+  | Ast.CeAtt(_loc, _attr, _payload, ce) ->
+     (* AFAICT there is no camlp4 grammar production constructing CeAtt.
+      * Nevertheless, I would expect CeAtt to be wrapped around classes
+      * as in "class a = object end [@foo]", therefore I recurse.
+      *)
+     tag_class_expr write_tag ce
 
   | <:class_expr< $virtual:_$ $id:_$ [ $_$ ] >>
     (* id on the right hand side -- ignore *)
@@ -559,6 +629,19 @@ let rec tag_let_pattern write_tag = function
     )
   | <:patt< ($p1$ : $_$) >> ->
     tag_let_pattern write_tag p1
+  | Ast.PaAtt(_loc, _attr, _payload, p) ->
+     (* Attributes can be wrapped around patterns, 
+      * eg "let x [@foo] = ..."
+      *)
+     tag_let_pattern write_tag p
+  | <:patt< exception $p$>> ->
+     (* Exception patterns are only permitted in proper matches with at 
+      * least 2 cases and not in let bindings. However, camlp4 parses 
+      * exception patterns in let bindings and only the type checker 
+      * gives an error on them. Therefore, do something here on exceptions 
+      * patterns.
+      *)
+     tag_let_pattern write_tag p
 
   | <:patt< $chr:_$ >>
   | <:patt< $int:_$ >>
@@ -590,7 +673,6 @@ let rec tag_let_pattern write_tag = function
     -> assert false
 
 
-
 let rec tag_str_item write_tag str_item = match str_item with
   | <:str_item< exception $exid$ >> 
   | <:str_item< exception $exid$ = $_$ >> ->
@@ -651,7 +733,7 @@ let rec tag_str_item write_tag str_item = match str_item with
       (Ast.list_of_module_binding mb [])
 
 
-  | <:str_item< open $_$ >>
+  | Ast.StOpn _ 			(* open x, open! x *)
   | <:str_item< include $_$ >>
   | <:str_item< $exp:_$ >>
   | <:str_item< # $_$ $_$ >>		(* directive *)
@@ -693,6 +775,12 @@ and tag_module_expr write_tag = function
     List.iter
       (tag_str_item write_tag)
       (Ast.list_of_str_item str_items [])
+  | Ast.MeAtt(_loc, _attr, _payload, me) ->
+     (* AFAICT there is no camlp4 grammar production constructing MeAtt.
+      * Nevertheless, I would expect MeAtt to be wrapped around signature
+      * types as in "module A = struct end [@foo]", therefore I recurse.
+      *)
+     tag_module_expr write_tag me
 
   | <:module_expr< $_$ $_$ >>		(* XXX check and probably tag this *)
   | <:module_expr< $id:_$ >>
